@@ -583,7 +583,20 @@ class SharedTensorBase:
         z = tf.strings.substr(z, 0, self.serialize_len)
         self.inner_value = tf.reshape(tf.io.parse_tensor(z, 'int64'), shape)
 
-
+    def load_from_fixed_length_file(self, path, header_bytes, footer_bytes, repeat, fields_num=None, batch_size=None):
+        if self.shape is None:
+            self.shape = [batch_size, fields_num]
+        elif fields_num is None and batch_size is None:
+            pass
+        else:
+            if self.shape != [batch_size, fields_num]:
+                raise Exception("must have self.shape == [batch_size, fields_num]")
+        dataset1 = tf.compat.v1.data.FixedLengthRecordDataset(
+            filenames=[path],
+            record_bytes=8 * np.prod(self.shape), header_bytes=header_bytes, footer_bytes=footer_bytes)
+        z = dataset1.repeat(repeat).make_one_shot_iterator().get_next()
+        zf = tf.io.decode_raw(z, "int64", little_endian=False)
+        self.inner_value = tf.reshape(zf, self.shape)
 
 
 def get_device(owner):
@@ -700,7 +713,7 @@ class PrivateTensorBase:
             if self.module is not None:
                 self.inner_value %= self.module
             if self.shape is not None:
-                if self.shape != self.inner_value.shape:
+                if self.shape != self.inner_value.shape.as_list():
                     raise StfEqualException("self.shape", "self.inner_value.shape",
                                             self.shape, self.inner_value.shape)
 
@@ -753,8 +766,13 @@ class PrivateTensorBase:
             data = clip(data)
             if map_fn is not None:
                 data = data.map(map_func=map_fn)
+            StfConfig.log_op_list.append(tf.print("load file "+path))
         self.load_from_tf_tensor(data)
 
+
+
+
+
     def __getitem__(self, item):
         with tf.device(self.owner):
             inner_value = self.inner_value.__getitem__(item)
@@ -802,6 +820,7 @@ class PrivateTensorBase:
             data = tf.reshape(data, [batch_size, output_col_num])
             data = tf.strings.to_number(data, out_type='float64')
             data = clip(data)
+            StfConfig.log_op_list.append(tf.print("load file " + path))
 
         self.load_from_tf_tensor(data)
         return idx_batch
@@ -823,6 +842,7 @@ class PrivateTensorBase:
             aline = tf.strings.to_number(aline, dtype)
             aline = tf.reshape(aline, [col_num])
             self.load_from_tf_tensor(aline)
+            StfConfig.log_op_list.append(tf.print("load file " + path))
 
     def to_tf_tensor(self, owner=None, dtype='float64') -> tf.Tensor:
         """
@@ -1295,6 +1315,7 @@ class SharedPairBase:
             x = self.to_private(owner)
             return x.to_tf_tensor()
 
+
     def ones_like(self):
         """
         See tf.ones_like
@@ -1468,6 +1489,16 @@ class SharedPairBase:
         with tf.device(StfConfig.workerR[0]):
             self.xR.unserialize(StfConfig.stf_home_workerR)
 
+    def load_from_fixed_length_file(self, pathL, pathR, header_bytes, footer_bytes, fields_num, batch_size, repeat):
+        with tf.device(StfConfig.workerL[0]):
+            if self.xL is None:
+                self.xL = SharedTensorBase(shape=[batch_size, fields_num])
+            self.xL.load_from_fixed_length_file(pathL, header_bytes, footer_bytes, repeat)
+        with tf.device(StfConfig.workerR[0]):
+            if self.xR is None:
+                self.xR = SharedTensorBase(shape=[batch_size, fields_num])
+            self.xR.load_from_fixed_length_file(pathR, header_bytes, footer_bytes, repeat)
+
     def random_uniform_adjoint(self, seed=None):
         with tf.device(self.ownerL):
             xL = self.xL.random_uniform_adjoint(seed)
